/************************************************************************
	display.c

    16x8 LED Display
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>
#include <string.h>

// Local includes
#include "hardware.h"
#include "display.h"

// Clear the LED display
//
// Note: this must be called before starting the display
// 		 update interrupt.
void displayClear(void)
{
	int loop;
	
	// Initialise the display state
	for (loop = 0; loop < 8; loop++)
	{
		display.row[loop].byte1 = 0x00;
		display.row[loop].byte2 = 0x00;
	}

	// Set the row to be displayed
	display.currentRow = 0;
}

void displayPlot(unsigned char x, unsigned char y, unsigned char state)
{
	// Check for out of bounds
	if (x > 15 || y > 7)
	{
		return;
	}

	// Read the byte at the current location
	if (x < 8)
	{
		if (state == ON) display.row[y].byte1 |= 0b10000000 >> x;
		else display.row[y].byte1 &= 0b01111111 >> x;
	}
	else
	{
		if (state == ON) display.row[y].byte2 |= 0b10000000 >> (x-8);
		else display.row[y].byte2 &= 0b01111111 >> (x-8);
	}
}

void displayLine(int x0, int y0, int x1, int y1, unsigned char state)
{
	int dy = y1 - y0;
	int dx = x1 - x0;
	int stepx, stepy;

	if (dy < 0)
	{
		dy = -dy; stepy = -1; 
	}
	else stepy = 1; 

 	if (dx < 0)
	{
		dx = -dx; stepx = -1; 
	}
	else stepx = 1; 

	dy <<= 1; 							// dy is now 2*dy
	dx <<= 1; 							// dx is now 2*dx
 
	displayPlot(x0, y0, state);

	if (dx > dy) {
		int fraction = dy - (dx >> 1);	// same as 2*dy - dx
		while (x0 != x1)
		{
			if (fraction >= 0)
			{
				y0 += stepy;
				fraction -= dx; 		// same as fraction -= 2*dx
			}

   			x0 += stepx;
   			fraction += dy; 				// same as fraction -= 2*dy
   			displayPlot(x0, y0, state);
		}
	}
	else
	{
		int fraction = dx - (dy >> 1);
		while (y0 != y1)
		{
			if (fraction >= 0)
			{
				x0 += stepx;
				fraction -= dy;
			}

			y0 += stepy;
			fraction += dx;
			displayPlot(x0, y0, state);
		}
	}
}

void displayRectangle(int x0, int y0, int x1, int y1, unsigned char state)
{
	displayLine(x0, y0, x0, y1, state);
	displayLine(x0, y1, x1, y1, state);
	displayLine(x1, y0, x1, y1, state);
	displayLine(x0, y0, x1, y0, state);
}

void displayFilledRectangle(int x0, int y0, int x1, int y1, unsigned char state)
{
	unsigned char x, y;

	// Plot the rectangle.  No need to use the line routine here since
	// direct plotting is quicker for straight lines.
	for (y = y0; y <= y1; y++)
		for (x = x0; x <= x1; x++)
			displayPlot(x, y, state);
}

void displayCircle(int xCentre, int yCentre, int radius, unsigned int state)
{
	int x = 0, y = radius;
	int d = 3 - (2 * radius);
 
    while(x <= y)
	{
		displayPlot(xCentre + x, yCentre + y, state);
		displayPlot(xCentre + y, yCentre + x, state);
		displayPlot(xCentre - x, yCentre + y, state);
		displayPlot(xCentre + y, yCentre - x, state);
		displayPlot(xCentre - x, yCentre - y, state);
		displayPlot(xCentre - y, yCentre - x, state);
		displayPlot(xCentre + x, yCentre - y, state);
		displayPlot(xCentre - y, yCentre + x, state);

		if (d < 0) d += (4 * x) + 6;
		else
		{
			d += (4 * (x - y)) + 10;
			y -= 1;
		}

		x++;
	}
}

void displayString(const char *tstring)
{
	int tempColumn = 0;
	int tempCharacter = 0;
	unsigned char x = 0;
	unsigned char y = 0;	
	unsigned char currentColumn;

	displayClear();

	// Plot a display full of text
	for (currentColumn = 0; currentColumn < 16; currentColumn++)
	{
		// Only plot if character is in range
		if (tempCharacter < strlen(tstring))
		{
			// Display a single column of 8 bits
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00000001) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00000010) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00000100) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00001000) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00010000) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b00100000) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b01000000) != 0) displayPlot(x, y, ON);
			y++;
			if ((Font8x5[tstring[tempCharacter]-32][tempColumn] & 0b10000000) != 0) displayPlot(x, y, ON);

			// Reset y and add one to x ready for the next column
			y -= 7;
			x++;
		}

		// Increment the column to be displayed
		tempColumn++;

		// End of character?
		if (tempColumn == 6)
		{
			// Next character
			tempColumn = 0;
			tempCharacter++;
		}
	}
}

void displayScrollText(const char *sstring)
{
	int temp;

	int character = 0;
	int column = 0;
	int currentColumn;
	int tempColumn;
	int tempCharacter;
	unsigned char x = 0;
	unsigned char y = 0;

	do
	{
		// Store the temp variables
		tempColumn = column;
		tempCharacter = character;

		// Reset x and y
		x = 0;
		y = 0;

		displayClear();

		// Plot a display full of text
		for (currentColumn = 0; currentColumn < 16; currentColumn++)
		{
			// Only plot if character is in range
			if (tempCharacter < strlen(sstring))
			{
				// Display a single column of 8 bits
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00000001) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00000010) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00000100) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00001000) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00010000) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b00100000) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b01000000) != 0) displayPlot(x, y, ON);
				y++;
				if ((Font8x5[sstring[tempCharacter]-32][tempColumn] & 0b10000000) != 0) displayPlot(x, y, ON);

				// Reset y and add one to x ready for the next column
				y -= 7;
				x++;
			}

			// Increment the column to be displayed
			tempColumn++;
	
			// End of character?
			if (tempColumn == 6)
			{
				// Next character
				tempColumn = 0;
				tempCharacter++;
			}
		}

		// Get ready for the next display full
		column++;
		if (column == 6)
		{
			column = 0;
			character++;
		}

		for (temp = 0; temp < 800; temp++) __delay_us(100);
	} while(character < strlen(sstring));
}

void displayNumber(unsigned char number)
{
	unsigned char number1;
	unsigned char number2;
	char noString[3];
	int temp;
	
	// Range check
	if (number > 99) number = 99;

	// Displays a number from 00-99 padded with a 0 if required
	if (number < 10) number1 = 0;
	else number1 = number / 10;

	number2 = number - (number1 * 10);

	// Generate a null terminated ASCII string
	noString[0] = number1 + 48;
	noString[1] = number2 + 48;
	noString[2] = '\0';

	displayString(noString);
}