// Main.c

// Global includes
#include <htc.h>
#include <stdio.h>
#include <string.h>

// Local includes
#include "hardware.h"
#include "display.h"
#include "interrupt.h"
#include "ds1302.h"
#include "lm35.h"
#include "input.h"

// PIC 18F2550 fuse configuration:
// Config word 1 (Oscillator configuration)
// 20Mhz crystal input scaled to 48Mhz and configured for USB operation
// from Table 2-3 page 30 of the datasheet
__CONFIG(1, USBPLL & IESODIS & FCMDIS & HSPLL & CPUDIV1 & PLLDIV5);
// Config word 2
__CONFIG(2, VREGEN & PWRTDIS & BORDIS & WDTDIS);
// Config word 3
__CONFIG(3, PBDIGITAL & LPT1DIS & MCLREN);
// Config word 4
__CONFIG(4, XINSTDIS & STVRDIS & LVPDIS & ICPORTDIS & DEBUGEN);
// Config word 5, 6 and 7 (protection configuration)
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

// Globals for holding strings
unsigned char message[256];
unsigned char tempString[64];

void pause(void)
{
	int loop;
	for (loop = 0; loop < 30000; loop++) __delay_us(100);
}

void main(void)
{
	int loop, length;
	unsigned char temperature;
	unsigned char daysInMonth;
	unsigned char number1, number2;

	unsigned char hours, minutes;

	// Enable ADC on AN0 (RA0)
	ADCON0 = 0b00000001; // Set the ADC running on AN0

	// Configure ADC on AN0 (RA0) and Digital IO on all other ports
	// Vss and Vdd as voltage references
	ADCON1 = 0b00001110;

	// Configure the ADC acquisition time according to the datasheet
	ADCON2 = 0b10110110; // Note: output is right justified

	// Set port directions
	TRISA = 0b00000001;
	TRISB = 0b00000000;
	TRISC = 0b01110000;

	// Clear all ports
	PORTA = 0b00000000;
	PORTB = 0b00010000;
	PORTC = 0b00000000;

	// Set up SPI
	SSPSTAT = 0b11000000;	// SPI Master
							// Tx occurs on idle to active transition

	SSPCON1 = 0b00100000;	// Serial port enabled
							// Speed is Fosc/4

	// Disable the USB device
	UCON = 0b00000000;
	UCFG = 0b00001000;

	// Clear the LED display
	displayClear();

	// Enable interrupts with priority
	IPEN = 1;

	// Set up interrupt on timer0 to update the display
	// 
	// Each call to the interrupt updates a single column
	// and we want each column to be lit for 200 microseconds...
	//
	// Fosc is 48,000,000 so Fosc/4 is 12,000,000 ticks/sec
	// meaning 200 microseconds is 200 * 12 = 2400 ticks
	// so with a 1:16 prescaler we need to interrupt every 150 ticks
	// which means TMR0L should be 256-150 = 106.
	TMR0IP = 0;			// Set timer0 interrupt to low priority
	TMR0IF = 0;			// Clear the timer0 interrupt flag
	TMR0L = 106;		// Reset the timer0 counter

	T0CON = 0b11000011; // Timer0 on, 8-bit and 1:16 prescaler
	TMR0IE = 1;			// Enable the timer0 interrupt

	// Enable interrupts
	GIEH = 1;			// Global enable all high priority interrupts
	GIEL = 1;			// Global enable all low priority interrupts

	// Initialise the Real Time Clock
	initRTC();

	// Get the current time and date to set the clock before running the
	// display loop
	displayScrollText("    Enter Hour:    ");
	datetime.hours = getNumber(0, 23);

	displayScrollText("    Enter Minutes:    ");
	datetime.minutes = getNumber(0,59);

	// Default seconds to 00
	datetime.seconds = 0;

	displayScrollText("    Enter Year:    ");
	datetime.year = getNumber(0,99);

	displayScrollText("    Enter Month:    ");
	datetime.month = getNumber(1,12);

	// Set the standard days in a month
	switch(datetime.month)
	{
		case 1: 	daysInMonth = 31;
					break;
		case 2: 	daysInMonth = 28;
					break;
		case 3: 	daysInMonth = 31;
					break;
		case 4: 	daysInMonth = 30;
					break;
		case 5: 	daysInMonth = 31;
					break;
		case 6: 	daysInMonth = 30;
					break;
		case 7: 	daysInMonth = 31;
					break;
		case 8: 	daysInMonth = 31;
					break;
		case 9: 	daysInMonth = 30;
					break;
		case 10: 	daysInMonth = 31;
					break;
		case 11: 	daysInMonth = 30;
					break;
		case 12: 	daysInMonth = 31;
					break;
	}

	// If the month is February, check for leap year
	if (datetime.month == 2 &&
		(datetime.year % 400 == 0 ||
		(datetime.year % 4 == 0 &&
		datetime.year % 100 != 0)))
	{
		daysInMonth = 29;
	}
		
	displayScrollText("    Enter Date:    ");
	datetime.day = getNumber(1, daysInMonth);

	displayScrollText("    Enter day:    ");
	datetime.dayNo = getDay();

/*
	datetime.hours = 8;
	datetime.minutes = 59;
	datetime.seconds = 45;
	datetime.year = 9;
	datetime.month = 10;
	datetime.day = 21;
	datetime.dayNo = 3;
*/

	// Set the RTC
	setRTC();

	while(1)
	{
		// Assemble the text to scroll across the display
		// The time is currently 00:00 on Sunday the 15th of Feburary 2009.  The temperature is 21DC.
		// There are 6 hour(s) and 54 minute(s) until 17:00 and 635 hours until 1700 on Friday.
		// (logo) TELIGENT Telecom - Adding Value to Your Network

		// Read the Real Time Clock
		readRTC();

		// Get the current temperature in degrees C
		temperature = readTemperature();

		// Blank the string
		message[0] = '\0';

		strcat(message, "    The time is currently ");
		timeString(tempString);
		strcat(message, tempString);

		strcat(message, " on ");
		dateString(tempString);
		strcat(message, tempString);

		strcat(message, ".    The temperature is ");

		if (temperature < 10) number1 = 0;
		else number1 = temperature / 10;
	
		number2 = temperature - (number1 * 10);
	
		if (number1 > 0)
		{
			length = strlen(message);
			message[length] = number1 + 48;
			message[length+1] = '\0';
		}
	
		length = strlen(message);
		message[length] = number2 + 48;
		message[length+1] = 129;
		message[length+2] = '\0';

		strcat(message, "C.    ");

		// If it's not the weekend and after 9 but before 17:00, display the time left until 17:00 today
		if (datetime.dayNo < 6 && datetime.hours > 8 && datetime.hours < 17)
		{
			hours = 17 - datetime.hours;

			if (hours == 1)
			{
				strcat(message, "There is less than 1 hour to COB today. ");
			}
			else
			{
				strcat(message, "There are less than ");
				if (hours < 10) number1 = 0;
				else number1 = hours / 10;
			
				number2 = hours - (number1 * 10);
			
				if (number1 > 0)
				{
					length = strlen(message);
					message[length] = number1 + 48;
					message[length+1] = '\0';
				}
			
				length = strlen(message);
				message[length] = number2 + 48;
				message[length+1] = '\0';
	
				strcat(message, " hours left until COB today. ");
			}
		}

		// Display the Teligent logo and strap line
		length = strlen(message);
		message[length] = 130;
		message[length+1] = 131;
		message[length+2] = 132;
		message[length+3] = '\0';
		strcat(message, " TELIGENT Telecom - Adding value to your network");

		strcat(message, "        ");

		// Display the message
		displayScrollText(message);
	}
}